@@ -49,11 +49,6 @@ static int checkSpec(rpmts ts, Header h)
     return rc;
 }
 
-/*
- * Kurwa, durni ameryka?ce sobe zawsze my?l?, ?e ca?y ?wiat mwi po
- * angielsku...
- */
-/* XXX this is still a dumb test but at least it's i18n aware */
 /**
  */
 static int isSpecFile(const char * specfile)
@@ -149,7 +144,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	(void) isCompressed(arg, &res);
 
 	cmd = alloca(strlen(arg) + 50 + strlen(tmpSpecFile));
-	sprintf(cmd, "%s < %s | tar xOvf - Specfile 2>&1 > %s",
+	sprintf(cmd, "%s < '%s' | tar xOvf - Specfile 2>&1 > '%s'",
 			zcmds[res & 0x3], arg, tmpSpecFile);
 	if (!(fp = popen(cmd, "r"))) {
 	    rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
@@ -161,7 +156,7 @@ static int buildForTarget(rpmts ts, const char * arg, BTA_t ba)
 	    /* Try again */
 	    (void) pclose(fp);
 
-	    sprintf(cmd, "%s < %s | tar xOvf - --wildcards \\*.spec 2>&1 > %s",
+	    sprintf(cmd, "%s < '%s' | tar xOvf - --wildcards \\*.spec 2>&1 > '%s'",
 		    zcmds[res & 0x3], arg, tmpSpecFile);
 	    if (!(fp = popen(cmd, "r"))) {
 		rpmError(RPMERR_POPEN, _("Failed to open tar pipe: %m\n"));
