@@ -451,7 +451,7 @@ static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data,
 		t = stpcpy(t, "RSA");
 		break;
 	    default:
-		snprintf(t, nb - (t - val), "%d", sigp->pubkey_algo);
+		(void) snprintf(t, nb - (t - val), "%d", sigp->pubkey_algo);
 		t += strlen(t);
 		break;
 	    }
@@ -466,7 +466,7 @@ static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data,
 		t = stpcpy(t, "SHA1");
 		break;
 	    default:
-		snprintf(t, nb - (t - val), "%d", sigp->hash_algo);
+		(void) snprintf(t, nb - (t - val), "%d", sigp->hash_algo);
 		t += strlen(t);
 		break;
 	    }
@@ -980,10 +980,12 @@ static int filerequireTag(Header h, /*@out@*/ rpmTagType * type,
 
 /* I18N look aside diversions */
 
+#if defined(ENABLE_NLS)
 /*@-exportlocal -exportheadervar@*/
 /*@unchecked@*/
-int _nl_msg_cat_cntr;	/* XXX GNU gettext voodoo */
+extern int _nl_msg_cat_cntr;	/* XXX GNU gettext voodoo */
 /*@=exportlocal =exportheadervar@*/
+#endif
 /*@observer@*/ /*@unchecked@*/
 static const char * language = "LANGUAGE";
 
@@ -1026,8 +1028,11 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	{   const char * tn = tagName(tag);
 	    const char * n;
 	    char * mk;
-	    (void) headerNVR(h, &n, NULL, NULL);
-	    mk = alloca(strlen(n) + strlen(tn) + sizeof("()"));
+	    size_t nb = sizeof("()");
+	    int xx = headerNVR(h, &n, NULL, NULL);
+	    if (tn)	nb += strlen(tn);
+	    if (n)	nb += strlen(n);
+	    mk = alloca(nb);
 	    sprintf(mk, "%s(%s)", n, tn);
 	    msgkey = mk;
 	}
@@ -1035,7 +1040,9 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	/* change to en_US for msgkey -> msgid resolution */
 	langval = getenv(language);
 	(void) setenv(language, "en_US", 1);
+#if defined(ENABLE_NLS)
 /*@i@*/	++_nl_msg_cat_cntr;
+#endif
 
 	msgid = NULL;
 	/*@-branchstate@*/
@@ -1052,7 +1059,9 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	    (void) setenv(language, langval, 1);
 	else
 	    unsetenv(language);
+#if defined(ENABLE_NLS)
 /*@i@*/	++_nl_msg_cat_cntr;
+#endif
 
 	if (domain && msgid) {
 	    *data = /*@-unrecog@*/ dgettext(domain, msgid) /*@=unrecog@*/;
