@@ -147,7 +147,6 @@ static Header rpmgiReadHeader(rpmgi gi, const char * path)
 /**
  * Read next header from package, lazily expanding manifests as found.
  * @todo An empty file read as manifest truncates argv returning RPMRC_NOTFOUND.
- * @todo Errors, e.g. non-existent path in manifest, will terminate iteration.
  * @todo Chained manifests lose an arg someplace.
  * @param gi		generalized iterator
  * @return		RPMRC_OK on success
@@ -311,12 +310,14 @@ static rpmRC rpmgiGlobArgv(rpmgi gi, /*@null@*/ ARGV_t argv)
 
     if (argv != NULL)
     while ((arg = *argv++) != NULL) {
+	char * t = rpmEscapeSpaces(arg);
 	ARGV_t av = NULL;
 
-	xx = rpmGlob(arg, &ac, &av);
+	xx = rpmGlob(t, &ac, &av);
 	xx = argvAppend(&gi->argv, av);
 	gi->argc += ac;
 	av = argvFree(av);
+	t = _free(t);
 	ac = 0;
     }
     return rpmrc;
@@ -458,6 +459,7 @@ rpmgi rpmgiNew(rpmts ts, int tag, const void * keyp, size_t keylen)
     gi->flags = 0;
     gi->active = 0;
     gi->i = -1;
+    gi->errors = 0;
     gi->hdrPath = NULL;
     gi->h = NULL;
 
@@ -565,12 +567,22 @@ rpmRC rpmgiNext(/*@null@*/ rpmgi gi)
 	    goto enditer;
 	}
 	break;
-    case RPMDBI_ARGLIST:
+    case RPMDBI_ARGLIST: 
 	/* XXX gi->active initialize? */
 if (_rpmgi_debug  < 0)
 fprintf(stderr, "*** gi %p\t%p[%d]: %s\n", gi, gi->argv, gi->i, gi->argv[gi->i]);
-	/* Read next header, lazily expanding manifests as found. */
-	rpmrc = rpmgiLoadReadHeader(gi);
+
+	/* 
+ 	 * Read next header, lazily expanding manifests as found,
+ 	 * count + skip errors.
+ 	 */
+	rpmrc = RPMRC_NOTFOUND;	
+	while (gi->i < gi->argc) {
+	    if ((rpmrc = rpmgiLoadReadHeader(gi)) == RPMRC_OK) 
+		break;
+	    gi->errors++;
+	    gi->i++;
+        }
 
 	if (rpmrc != RPMRC_OK)	/* XXX check this */
 	    goto enditer;
@@ -691,4 +703,9 @@ rpmRC rpmgiSetArgs(rpmgi gi, ARGV_t argv, int ftsOpts, rpmgiFlags flags)
     return rpmgiGlobArgv(gi, argv);
 }
 
+int rpmgiNumErrors(rpmgi gi)
+{
+    return (gi != NULL ? gi->errors : -1);
+}
+
 /*@=modfilesys@*/
