@@ -476,8 +476,8 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
 	str = PyString_AsString(item);
 	while (extensions->name) {
 	    if (extensions->type == HEADER_EXT_TAG
-		&& !xstrcasecmp(extensions->name + 7, str)) {
-		(const struct headerSprintfExtension *) ext = extensions;
+	     && !xstrcasecmp(extensions->name + 7, str)) {
+		ext = extensions;
 	    }
 	    extensions++;
 	}
@@ -537,6 +537,8 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     case RPMTAG_CONFLICTNAME:
     case RPMTAG_CONFLICTFLAGS:
     case RPMTAG_CONFLICTVERSION:
+    case RPMTAG_CHANGELOGTIME:
+    case RPMTAG_FILEVERIFYFLAGS:
 	forceArray = 1;
 	break;
     case RPMTAG_SUMMARY:
@@ -1063,6 +1065,9 @@ PyObject * labelCompare (PyObject * self, PyObject * args)
 			&e1, &v1, &r1, &e2, &v2, &r2))
 	return NULL;
 
+    if (e1 == NULL)	e1 = "0";
+    if (e2 == NULL)	e2 = "0";
+
     rc = compare_values(e1, e2);
     if (!rc) {
 	rc = compare_values(v1, v2);
