@@ -439,7 +439,7 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 	    if (p->type == RPMPROB_BADRELOCATE)
 		continue;
 
-	    byName = p->pkgNEVR;
+	    byName = strdup(p->pkgNEVR);
 	    if ((byArch= strrchr(byName, '.')) != NULL)
 		*byArch++ = '\0';
 	    if ((byRelease = strrchr(byName, '-')) != NULL)
@@ -475,6 +475,7 @@ fprintf(stderr, "*** rpmts_Check(%p) ts %p cb %p\n", s, s->ts, cbInfo.cb);
 #endif
 	    PyList_Append(list, (PyObject *) cf);
 	    Py_DECREF(cf);
+	    free(byName);
 	}
 
 	ps = rpmpsFree(ps);
@@ -890,6 +891,14 @@ fprintf(stderr, "*** rpmts_SetVSFlags(%p) ts %p\n", s, s->ts);
  */
 /*@null@*/
 static PyObject *
+rpmts_GetVSFlags(rpmtsObject * s)
+{
+    return Py_BuildValue("i", rpmtsVSFlags(s->ts));
+}
+
+/** \ingroup py_c
+ */
+static PyObject *
 rpmts_SetColor(rpmtsObject * s, PyObject * args, PyObject * kwds)
 	/*@modifies s @*/
 {
@@ -1476,6 +1485,9 @@ static struct PyMethodDef rpmts_methods[] = {
     rpm.RPMVSF_NORSA         if set, don't check header+payload RSA signature\n\
     rpm._RPMVSF_NODIGESTS    if set, don't check digest(s)\n\
     rpm._RPMVSF_NOSIGNATURES if set, don't check signature(s)\n" },
+ {"getVSFlags",(PyCFunction) rpmts_GetVSFlags,	METH_NOARGS,
+"ts.getVSFlags() -> vsflags\n\
+- Retrieve current signature verification flags from transaction\n" },
  {"setColor",(PyCFunction) rpmts_SetColor,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
  {"pgpPrtPkts",	(PyCFunction) rpmts_PgpPrtPkts,	METH_VARARGS|METH_KEYWORDS,
