@@ -47,7 +47,9 @@ hashBucket findEntry(hashTable ht, const void * key)
 
     /*@-modunconnomods@*/
     hash = ht->fn(key) % ht->numBuckets;
+/*@-boundsread@*/
     b = ht->buckets[hash];
+/*@=boundsread@*/
 
     while (b && b->key && ht->eq(b->key, key))
 	b = b->next;
@@ -72,10 +74,12 @@ unsigned int hashFunctionString(const void * string)
     const char * chp = string;
 
     len = strlen(string);
+/*@-boundsread@*/
     for (i = 0; i < len; i++, chp++) {
 	xorValue ^= *chp;
 	sum += *chp;
     }
+/*@=boundsread@*/
 
     return ((((unsigned)len) << 16) + (((unsigned)sum) << 8) + xorValue);
 }
@@ -98,6 +102,7 @@ hashTable htCreate(int numBuckets, int keySize, int freeData,
     return ht;
 }
 
+/*@-boundswrite@*/
 void htAddEntry(hashTable ht, const void * key, const void * data)
 {
     unsigned int hash;
@@ -129,25 +134,32 @@ void htAddEntry(hashTable ht, const void * key, const void * data)
     b->data = xrealloc(b->data, sizeof(*b->data) * (b->dataCount + 1));
     b->data[b->dataCount++] = data;
 }
+/*@=boundswrite@*/
 
-void htFree(hashTable ht)
+hashTable htFree(hashTable ht)
 {
     hashBucket b, n;
     int i;
 
     for (i = 0; i < ht->numBuckets; i++) {
+/*@-boundsread@*/
 	b = ht->buckets[i];
+/*@=boundsread@*/
 	if (b == NULL)
 	    continue;
+/*@-boundswrite@*/
 	ht->buckets[i] = NULL;
+/*@=boundswrite@*/
 	if (ht->keySize > 0)
 	    b->key = _free(b->key);
 	do {
 	    n = b->next;
 	    /*@-branchstate@*/
 	    if (b->data) {
+/*@-boundswrite@*/
 		if (ht->freeData)
 		    *b->data = _free(*b->data);
+/*@=boundswrite@*/
 		b->data = _free(b->data);
 	    }
 	    /*@=branchstate@*/
@@ -157,6 +169,7 @@ void htFree(hashTable ht)
 
     ht->buckets = _free(ht->buckets);
     ht = _free(ht);
+    return NULL;
 }
 
 int htHasEntry(hashTable ht, const void * key)
@@ -174,12 +187,14 @@ int htGetEntry(hashTable ht, const void * key, const void *** data,
     if ((b = findEntry(ht, key)) == NULL)
 	return 1;
 
+/*@-boundswrite@*/
     if (data)
 	*data = (const void **) b->data;
     if (dataCount)
 	*dataCount = b->dataCount;
     if (tableKey)
 	*tableKey = b->key;
+/*@=boundswrite@*/
 
     return 0;
 }
