@@ -617,8 +617,11 @@ doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
 
     if ((shf = popen(pcmd, "r")) == NULL)
 	return 1;
-    while(mb->nb > 0 && (c = fgetc(shf)) != EOF)
-	SAVECHAR(mb, c);
+    while((c = fgetc(shf)) != EOF) {
+	if (mb->nb > 1) {
+	    SAVECHAR(mb, c);
+	} 
+    }
     (void) pclose(shf);
 
     /* XXX delete trailing \r \n */
@@ -1163,23 +1166,24 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    be++;
 /*@=globs@*/
 	*be++ = '\0';
-#ifndef	DEBUG_MACROS
 	(void) isCompressed(b, &compressed);
-#endif
 	switch(compressed) {
 	default:
-	case 0:	/* COMPRESSED_NOT */
+	case COMPRESSED_NOT:
 	    sprintf(be, "%%_cat %s", b);
 	    break;
-	case 1:	/* COMPRESSED_OTHER */
+	case COMPRESSED_OTHER:
 	    sprintf(be, "%%_gzip -dc %s", b);
 	    break;
-	case 2:	/* COMPRESSED_BZIP2 */
+	case COMPRESSED_BZIP2:
 	    sprintf(be, "%%_bzip2 %s", b);
 	    break;
-	case 3:	/* COMPRESSED_ZIP */
+	case COMPRESSED_ZIP:
 	    sprintf(be, "%%_unzip %s", b);
 	    break;
+        case COMPRESSED_LZMA:
+            sprintf(be, "%%_lzma -dc %s", b);
+            break;
 	}
 	b = be;
     } else if (STREQ("S", f, fn)) {
@@ -1698,6 +1702,8 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     int argc = 0;
     const char ** argv = NULL;
     char * globRoot = NULL;
+    const char *home = getenv("HOME");
+    int gflags = 0;
 #ifdef ENABLE_NLS
     const char * old_collate = NULL;
     const char * old_ctype = NULL;
@@ -1707,6 +1713,9 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     int i, j;
     int rc;
 
+    if (home != NULL && strlen(home) > 0) 
+	gflags |= GLOB_TILDE;
+
     rc = XpoptParseArgvString(patterns, &ac, &av);
     if (rc)
 	return rc;
@@ -1741,7 +1750,7 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, argv[argc]);
 	
 	gl.gl_pathc = 0;
 	gl.gl_pathv = NULL;
-	rc = Glob(av[j], GLOB_TILDE, Glob_error, &gl);
+	rc = Glob(av[j], gflags, Glob_error, &gl);
 	if (rc)
 	    goto exit;
 
@@ -2060,13 +2069,22 @@ rpmFreeMacros(MacroContext mc)
 }
 /*@=globstate@*/
 
+static int rpmFileHasSuffix(const char *path, const char *suffix)
+{
+    size_t plen = strlen(path);
+    size_t slen = strlen(suffix);
+    return (plen >= slen &&
+            strcmp(path+plen-slen, suffix) == 0);
+}
+
+
 /* =============================================================== */
 int isCompressed(const char * file, rpmCompressedMagic * compressed)
 {
     FD_t fd;
     ssize_t nb;
     int rc = -1;
-    unsigned char magic[4];
+    unsigned char magic[13];
 
     *compressed = COMPRESSED_NOT;
 
@@ -2097,6 +2115,11 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
     } else if ((magic[0] == 0120) && (magic[1] == 0113) &&
 	 (magic[2] == 0003) && (magic[3] == 0004)) {	/* pkzip */
 	*compressed = COMPRESSED_ZIP;
+    } else if ((magic[0] == 0xff) && (magic[1] == 0x4c) &&
+	       (magic[2] == 0x5a) && (magic[3] == 0x4d) &&
+	       (magic[4] == 0x41) && (magic[5] == 0x00)) {
+	/* new style lzma with magic */
+	*compressed = COMPRESSED_LZMA;
     } else if (((magic[0] == 0037) && (magic[1] == 0213)) || /* gzip */
 	((magic[0] == 0037) && (magic[1] == 0236)) ||	/* old gzip */
 	((magic[0] == 0037) && (magic[1] == 0036)) ||	/* pack */
@@ -2104,6 +2127,8 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
 	((magic[0] == 0037) && (magic[1] == 0235))	/* compress */
 	) {
 	*compressed = COMPRESSED_OTHER;
+    } else if (rpmFileHasSuffix(file, ".lzma")) {
+	*compressed = COMPRESSED_LZMA;
     }
 
     return rc;
