@@ -342,7 +342,7 @@ FD_t XfdNew(const char * msg, const char * file, unsigned line)
     fd->fps[0].fdno = -1;
 
     fd->url = NULL;
-    fd->rd_timeoutsecs = 1;	/* XXX default value used to be -1 */
+    fd->rd_timeoutsecs = 60;	/* XXX default value used to be -1 */
     fd->contentLength = fd->bytesRemain = -1;
     fd->wr_chunked = 0;
     fd->syserrno = 0;
@@ -2325,7 +2325,7 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 	fd = fdOpen(path, flags, mode);
 	if (fd) {
 	    fdSetIo(fd, ufdio);
-	    fd->rd_timeoutsecs = 1;
+	    fd->rd_timeoutsecs = 60;
 	    fd->contentLength = fd->bytesRemain = -1;
 	}
 	break;
@@ -3340,6 +3340,31 @@ int Fcntl(FD_t fd, int op, void *lip)
 /* =============================================================== */
 /* Helper routines that may be generally useful.
  */
+
+char * rpmEscapeSpaces(const char * s)
+{
+    const char * se;
+    char * t;
+    char * te;
+    size_t nb = 0;
+
+    for (se = s; *se; se++) {
+        if (isspace(*se))
+            nb++;
+        nb++;
+    }
+    nb++;
+
+    t = te = xmalloc(nb);
+    for (se = s; *se; se++) {
+        if (isspace(*se))
+            *te++ = '\\';
+        *te++ = *se;
+    }
+    *te = '\0';
+    return t;
+}
+
 /*@-bounds@*/
 int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
 {
