@@ -5,6 +5,9 @@
  * \file rpmio/rpmmacro.h
  */
 
+#include <stdio.h>
+#include <stdlib.h>
+
 /*! The structure used to store a macro. */
 typedef /*@abstract@*/ struct MacroEntry_s {
     struct MacroEntry_s *prev;/*!< Macro entry stack. */
@@ -172,7 +175,8 @@ typedef enum rpmCompressedMagic_e {
     COMPRESSED_NOT		= 0,	/*!< not compressed */
     COMPRESSED_OTHER		= 1,	/*!< gzip can handle */
     COMPRESSED_BZIP2		= 2,	/*!< bzip2 can handle */
-    COMPRESSED_ZIP		= 3	/*!< unzip can handle */
+    COMPRESSED_ZIP		= 3,	/*!< unzip can handle */
+    COMPRESSED_LZMA		= 4	/*!< lzma can handle */
 } rpmCompressedMagic;
 
 /**
