@@ -1,5 +1,5 @@
 /**
- * \file lib/stubs.c
+ * \file rpmio/stubs.c
  */
 
 /* XXX Portable shared libraries require rpmlib to contain these functions. */
@@ -71,6 +71,8 @@
 #endif
 
 #if defined(USE_GNU_GLOB)
+#include "misc/fnmatch.h"
 #include "misc/fnmatch.c"
+#include "misc/glob.h"
 #include "misc/glob.c"
 #endif
