@@ -64,6 +64,8 @@ enum modes {
 #define	MODES_FOR_TEST		(MODES_BT | MODES_IE)
 #define	MODES_FOR_ROOT		(MODES_BT | MODES_IE | MODES_QV | MODES_DB | MODES_K)
 
+static int quiet;
+
 /* the structure describing the options we take and the defaults */
 /*@unchecked@*/
 static struct poptOption optionsTable[] = {
@@ -109,6 +111,8 @@ static struct poptOption optionsTable[] = {
 	NULL },
 #endif	/* IAM_RPMEIU */
 
+ { "quiet", '\0', 0, &quiet, 0,			NULL, NULL},
+
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliAllPoptTable, 0,
 	N_("Common options for all rpm modes and executables:"),
 	NULL },
@@ -524,6 +528,9 @@ int main(int argc, const char ** argv)
 	}
     }
 
+    if (quiet)
+	rpmSetVerbosity(RPMLOG_WARNING);
+
 #if defined(IAM_RPMBT) || defined(IAM_RPMK)
     if (0
 #if defined(IAM_RPMBT)
@@ -681,7 +688,7 @@ int main(int argc, const char ** argv)
     case MODE_BUILD:
     case MODE_TARBUILD:
     {	const char * pkg;
-        while (!rpmIsVerbose())
+        if (!quiet) while (!rpmIsVerbose())
 	    rpmIncreaseVerbosity();
        
 	switch (ba->buildChar) {
@@ -859,7 +866,7 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 #endif
     case MODE_UNKNOWN:
 	if (poptPeekArg(optCon) != NULL || argc <= 1 || rpmIsVerbose()) {
-	    printUsage(optCon, stdout, 0);
+	    printUsage(optCon, stderr, 0);
 	    ec = argc;
 	}
 	break;
@@ -910,8 +917,8 @@ exit:
     /*@=noeffect@*/
 #endif
 
-    /* XXX don't overflow single byte exit status */
-    if (ec > 255) ec = 255;
+    /* XXX Avoid exit status overflow. Status 255 is special to xargs(1) */
+    if (ec > 254) ec = 254;
 
     /*@-globstate@*/
     return ec;
