@@ -1,7 +1,11 @@
 #include "system.h"
+const char *__progname;
 
 #include <rpmlib.h>
 #include "header_internal.h"
+
+#include "rpmdb.h"
+
 #include "debug.h"
 
 int main(int argc, char ** argv)
